#include <OneWire.h>
#include <RCSwitch.h>

boolean Debug = false;


OneWire  ds1(7);
OneWire  ds2(8);
RCSwitch mySwitch = RCSwitch();

#define PwmOut1  3
#define PwmOut2  5
#define PwmOut3  6
#define AcOut    9
#define LedOut   13

#define CodeBtn1 11556
#define CodeBtn2 11558
#define CodeBtn3 11557
#define MaxTemp  45

uint16_t curOut1=0,   curOut2=0,   curOut3=0;
uint16_t maxOut1=255, maxOut2=255, maxOut3=255;
boolean powerOut1=false,powerOut2=false,powerOut3=false;

void setup() {               
  // Инициализируем цифровой вход/выход в режиме выхода.
  // Выход 13 на большинстве плат Arduino подключен к светодиоду на плате.
  if( Debug ){
     Serial.begin(9600);
     Serial.println("Led controller started...");
  }
  mySwitch.enableReceive(0);  // Receiver on inerrupt 0 => that is pin #2

  pinMode(LedOut,  OUTPUT);
  pinMode(AcOut,   OUTPUT);
  pinMode(PwmOut1, OUTPUT);
  
  
  pinMode(PwmOut2, OUTPUT);
  pinMode(PwmOut3, OUTPUT);
  
  analogWrite(PwmOut1,255);
  analogWrite(PwmOut2,255);
  analogWrite(PwmOut3,255);
  digitalWrite(AcOut,true); 
  
}
 
boolean x = true; 

unsigned long ms1 = 0; 
unsigned long ms2 = 0;
unsigned long ms3 = 0;
unsigned long ms4 = 0;


int value         = 0;
boolean rc_flag   = false;
boolean long_flag = false;

void loop() {
   unsigned long ms = millis();
   if (mySwitch.available()) {
       value = mySwitch.getReceivedValue();
       if( /*value == CodeBtn1 ||*/
           value == CodeBtn2 ||
           value == CodeBtn3 ){
       
          if( !rc_flag )ms1 = ms;
          ms2 = ms;
          rc_flag = true;
          if( Debug ){
             Serial.print("Receive code ");
             Serial.print(ms-ms1);
             Serial.print(" ");
             Serial.print(ms-ms2);
             Serial.print(" ");
             Serial.println(value);
          }
       }//end if value
       mySwitch.resetAvailable();
   }

// Длинное нажатие клавиши (срабатывет один раз)
   if( rc_flag && (ms-ms1) > 2000  ){
      if( !long_flag )EventCode2();
      long_flag = true;
   }
   // Клавишу наконец то отпустили
   if( rc_flag && (ms-ms1) > 200 && (ms-ms2) > 100 ){
      if( !long_flag)EventCode1();
      rc_flag = false;
      long_flag = false;
   }
   
   
   SetPowerAC();
   SetPowerDC();
    
   if( (ms - ms3) > 60000 || ms3 > ms){
      ms3 = ms;
      CheckTemperature();
   } 
   
   
    mySwitch.resetAvailable();   
    delay(7); 
  
  
}

/**
* Проверяем температуру радиаторов
*/
void CheckTemperature(){
   float   t1  = GetDS18X20(ds1);
   float   t2  = GetDS18X20(ds2);
   if( Debug ){
      Serial.print("T1=");
      Serial.print(t1,1);
      Serial.print(",T2=");
      Serial.print(t2,1);
      Serial.println("");
   }   
   if( t1 >= MaxTemp && maxOut1 == 255 ){
      maxOut3 = 127;
      DebugPrint1();
   }
   if( t1 < MaxTemp && maxOut1 !=255 ) {
      maxOut3 = 255;
      DebugPrint1();
   }
   if( t2 >= MaxTemp && maxOut2 == 255 ){
      maxOut1 = 127;
      maxOut2 = 127;
      DebugPrint1();
  }
   if( t2 < MaxTemp && maxOut2 !=255 ){
      maxOut1 = 255;
      maxOut2 = 255;
      DebugPrint1();
   }
   
   if( maxOut1 < 255 || maxOut2 < 255 || maxOut3 < 255 )digitalWrite(13,HIGH);
   else digitalWrite(13,LOW);
}

/**
* Реакция на короткое нажатие клавиши
*/
void EventCode1(){
   if( value ){
      if( Debug ){
         Serial.print("Short press key ");
         Serial.println(value);
      }
      switch( value ){
//         case CodeBtn1 :
//         case 20483 :
//            powerOut1 = !powerOut1; 
//            DebugPrint1();
//            break;
         case CodeBtn2 :
//         case 21248 : 
            powerOut2 = !powerOut2;
            DebugPrint1(); 
            break;
          case CodeBtn3 : 
//         case 23552:
            powerOut3 = !powerOut3;
            powerOut1 = !powerOut1; 
            DebugPrint1(); 
            break;
       }
    }
    value = 0;
}

/**
* Реакция на длинное нажатие клавиши
*/
void EventCode2(){
   if( Debug ){
      Serial.print("Long press key ");
      Serial.println(value);
   }
   if( powerOut1 || powerOut2 || powerOut3 ){
      powerOut1 = false;
      powerOut2 = false;
      powerOut3 = false;
   }
   else {
      powerOut1 = true;
      powerOut2 = true;
      powerOut3 = true;
   }
}



void DebugPrint1(){
  if( Debug ){
     Serial.print("ch1=");
     Serial.print(powerOut1);  
     Serial.print(",");
     Serial.print(maxOut1);  
     Serial.print(",");
     Serial.print(curOut1);  
     Serial.print("  ch2=");
     Serial.print(powerOut2);  
     Serial.print(",");
     Serial.print(maxOut2);  
     Serial.print(",");
     Serial.print(curOut2);  
     Serial.print("  ch3=");
     Serial.print(powerOut3);  
     Serial.print(",");
     Serial.print(maxOut3);  
     Serial.print(",");
     Serial.println(curOut3);  
  }
}

// Включаем/отключаем 220В у блока питания   
void SetPowerAC(){
     if( powerOut1 || powerOut2 || powerOut3 ){
      digitalWrite(AcOut,true); 
   }
   else {
      if( curOut1 == 0 && curOut2 == 0 && curOut3 == 0 )digitalWrite(AcOut,false); 
   }
}

/**
* Плавное включение/отключение светожиодов по нискому напряжению
*/
void SetPowerDC(){
   if( powerOut1 == true && curOut1 < maxOut1 ){
       curOut1++;
       analogWrite(PwmOut1,255-curOut1);
   }
   if( powerOut1 == true && curOut1 > maxOut1 ){
       curOut1--;
       analogWrite(PwmOut1,255-curOut1);
   }
   if( powerOut1 == false && curOut1 > 0){
       curOut1--;
       analogWrite(PwmOut1,255-curOut1);
   }

   if( powerOut2 == true && curOut2 < maxOut2 ){
       curOut2++;
       analogWrite(PwmOut2,255-curOut2);
   }
   if( powerOut2 == true && curOut2 > maxOut2 ){
       curOut2--;
       analogWrite(PwmOut2,255-curOut2);
   }
   if( powerOut2 == false && curOut2 > 0){
       curOut2--;
       analogWrite(PwmOut2,255-curOut2);
   }

   if( powerOut3 == true && curOut3 < maxOut3 ){
       curOut3++;
       analogWrite(PwmOut3,255-curOut3);
   }
   if( powerOut3 == true && curOut3 > maxOut3 ){
       curOut3--;
       analogWrite(PwmOut3,255-curOut3);
   }
   if( powerOut3 == false && curOut3 > 0){
       curOut3--;
       analogWrite(PwmOut3,255-curOut3);
   }
}


/**
* Опрос датчика температуры
*/

float  GetDS18X20(OneWire ds){
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius;
  ds.reset_search();
  if ( !ds.search(addr)) {
    Serial.println("No DS18X20 sensors.");
    Serial.println();
    ds.reset_search();
    delay(250);
    return -99;
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return -99;
  }
 
  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
//      Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
//      Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
//      Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
//      Serial.println("Device is not a DS18x20 family device.");
      return 0;
  } 

  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
  delay(1000);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
  }
  celsius = (float)raw / 16.0;
//  Serial.print("  Temperature = ");
//  Serial.print(celsius);
//  Serial.println(" Celsius, ");
  return( celsius);

}

